#include <iostream>
#include <string>
using namespace std;

#include "console.h"
#include "simpio.h"
#include "stack.h"

/**
 * Returns true iff the provided character is one of
 * the binary arithmetic operators we've elected to support.
 */
static bool isSupportedOperator(char op) {
    switch (op) {
        case '+': case '-': case '*': case '/': case '%':
            return true;
        default:
            return false;
    }
}

/**
 * Computes and returns the integer that results when the integer
 * values identified via first and second are combined using
 * the supplied (presumably legitimate) operator.
 */
static int compute(int first, char op, int second) {
    switch (op) {
        case '+': return first + second;
        case '-': return first - second;
        case '*': return first * second;
        case '/': return first / second;
        case '%': return first % second;
    }

    error("Operator isn't supported.. Aborting...");
    return 0; // some compilers don't know that error kills the program
}

/**
 * Accepts the instruction (e.g. a string of the form "48+9-13**") and
 * the provided instruction as an RPN expression, using a stack for storage.
 * If all goes well, then the stack contains just one integer after
 * it's all over, and that integer is the result of the computation.
 * We return true if and only if the computation was executed without drama.
 */
static int evaluate(const string& instructions) {
    Stack<int> memory;
    for (size_t i = 0; i < instructions.size(); i++) {
        char inst = instructions[i];
        if (isdigit(inst)) {
            memory.push(inst - '0');
        } else if (isSupportedOperator(inst)) {
            if (memory.size() < 2) error("Too few digits for operator");
            int second = memory.pop();
            int first = memory.pop();
            int result = compute(first, inst, second);
            memory.push(result);
        } else {
            error("Invalid character in instruction string");
        }
    }

    if (memory.size() > 1) error("Too many digits in result");
    return memory.pop();
}

/**
 * Interactive read-eval-print loop in place to exercise
 * the above functionality.
 */
int main() {
    setConsoleFont("Courier-BOLD-24");
    cout << "Welcome to our RPN calculator.  You'll love this." << endl;
    while (true) {
        string response = getLine("Data sequence? ");
        if (response == "") break;
        try {
            int result = evaluate(response);
            cout << response << " evaluates to " << result << "." << endl;
        } catch (ErrorException ex) {
            cout << "There were problems with your instruction sequence." << endl;
            cout << "Error: " << ex.getMessage() << endl;
            cout << "Please try again." << endl;
        }
    }
    return 0;
}
